#!/usr/bin/env python
#-*- coding:utf-8 -*-
#===============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================================================

""" System API. """

from schtob.pyontapi.api.base import BaseAPI, NaArgument, NaField, NaTypeElement
from schtob.pyontapi.api.base import NaNamedType

class System(BaseAPI):
    
    """ System API class for base system calls. """
    
    __api_name__ = 'system'
    
    def get_ontapi_version(self):
        """ invoke `system-get-ontapi-version`. """
        fields = (
            NaField({'name': u'major-version', 'type': int}), 
            NaField({'name': u'minor-version', 'type': int}),
        )
        return self._filer.do_api_call(u'system-get-ontapi-version', (), fields)
    
    def api_get_elements(self, api_list):
        """ invoke `system-api-get-elements`. """
        args = (
            NaArgument(api_list, {
                'name': u'api-list',
                'type': NaNamedType(u'api-list-info', [
                            NaTypeElement({'name': '', 'type': unicode})
                        ]),
                'is-array': True,
            }),
        )
        fields = (
            NaField({'name': u'api-entries', 'is-array': True, 'type': 
                NaNamedType(u'system-api-entry-info', [
                    NaTypeElement({'name': u'name', 'type': unicode}),
                    NaTypeElement({'name': u'api-elements', 'is-array': True, 
                        'type': NaNamedType(u'system-api-element-info', [
                            NaTypeElement({
                                'name': u'encrypted',
                                'type': unicode,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': u'is-nonempty',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': u'is-optional',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': u'is-output',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': u'is-validated',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': u'name',
                                'type': unicode,
                            }), NaTypeElement({
                                'name': u'type',
                                'type': unicode,
                            }),
                    ])})
            ])}),
        )
        return self._filer.do_api_call(u'system-api-get-elements', args, fields)
    
    def api_list(self):
        """ invoke `system-api-list`. """
        fields = (
            NaField({
                'name': u'apis', 'is-array': True,
                'type': NaNamedType(u'system-api-info', [
                    NaTypeElement({
                        'name': u'is-streaming',
                        'type': bool,
                        'is-optional': True,
                    }),
                    NaTypeElement({
                        'name': u'license',
                        'type': unicode,
                        'is-optional': True,
                    }),
                    NaTypeElement({
                        'name': u'name',
                        'type': unicode,
                    }),
                ]),
            }),
        )
        return self._filer.do_api_call(u'system-api-list', (), fields)
    
    def get_api_list_types(self):
        """ invoke `system-api-list-types`. """
        fields = (
            NaField({'name': u'type-entries', 'is-array': True, 
                'type': NaNamedType(u'system-api-type-entry-info', [
                NaTypeElement({'name': u'name', 'type': unicode}),
                NaTypeElement({'name': u'type-elements', 'is-array': True, 
                  'type': NaNamedType(u'system-api-element-info', [
                    NaTypeElement({
                        'name': u'encrypted',
                        'type': unicode,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': u'is-nonempty',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': u'is-optional',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': u'is-output',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': u'is-validated',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': u'name',
                        'type': unicode,
                    }), NaTypeElement({
                        'name': u'type',
                        'type': unicode,
                    }),
                ])}),
            ])}),
        )
        return self._filer.do_api_call(u'system-api-list-types', (), fields)
